#ifndef YOLOV5CLS_H
#define YOLOV5CLS_H
#include "interfaceyoloengine.h"
#include "cuda_utils.h"

#include "opencv2/core.hpp"

class yolov5cls : public interfaceyoloengine
{
public:
    yolov5cls();
    ~yolov5cls();
    int createngine(std::string &wts_name, std::string& engine_name, char s_value) override;
    int setengine(std::string& engine_name) override;
    std::vector<std::vector<int>> detect(cv::Mat& image);
    int kClsInputH = 64;
    int kClsInputW = 64;
    int kClsNumClass = 2;
    int kOutputSize = kClsNumClass;

private:
    void serialize_engine(unsigned int max_batchsize, float& gd, float& gw, std::string& wts_name, std::string& engine_name);
    void deserialize_engine(std::string& engine_name, nvinfer1::IRuntime** runtime, nvinfer1::ICudaEngine** engine, nvinfer1::IExecutionContext** context);
    void prepare_buffers(nvinfer1::ICudaEngine* engine, float** gpu_input_buffer, float** gpu_output_buffer, float** cpu_input_buffer, float** cpu_output_buffer);
    void batch_preprocess(std::vector<cv::Mat>& imgs, float* output);
    void infer(nvinfer1::IExecutionContext& context, cudaStream_t& stream, void **buffers, float* input, float* output, int batchSize);
    std::vector<int> topk(const std::vector<float>& vec, int k);
    std::vector<float> softmax(float *prob, int n);
    cudaStream_t stream;
    float* gpu_buffers[2];
    float* cpu_input_buffer = nullptr;
    float* cpu_output_buffer = nullptr;
    nvinfer1::IRuntime* runtime = nullptr;
    nvinfer1::ICudaEngine* engine = nullptr;
    nvinfer1::IExecutionContext* context = nullptr;

};

#endif // YOLOV5CLS_H
