#ifndef YOLOV5DET_H
#define YOLOV5DET_H
#include "interfaceyoloengine.h"
#include "types.h"
#include "cuda_utils.h"
#include "preprocess.h"
#include "postprocess.h"

#include <vector>
#include "opencv2/core.hpp"

class yolov5det : public interfaceyoloengine
{
public:
    yolov5det();
    ~yolov5det();
    int createngine(std::string &wts_name, std::string& engine_name, char s_value) override;
    int setengine(std::string& engine_name) override;
    std::vector<std::vector<Detection>> detect(cv::Mat& image);
    int kInputH = 96;
    int kInputW = 192;
    int kNumClass = 12;
    int kMaxNumOutputBbox = 1000;
    float kConfThresh = 0.7f;
    float kNmsThresh = 0.45f;

private:
    void serialize_engine(unsigned int max_batchsize, bool& is_p6, float& gd, float& gw, std::string& wts_name, std::string& engine_name);
    void deserialize_engine(std::string& engine_name, nvinfer1::IRuntime** runtime, nvinfer1::ICudaEngine** engine, nvinfer1::IExecutionContext** context);
    void prepare_buffers(nvinfer1::ICudaEngine* engine, float** gpu_input_buffer, float** gpu_output_buffer, float** cpu_output_buffer);
    void infer(nvinfer1::IExecutionContext& context, cudaStream_t& stream, void** gpu_buffers, float* output, int batchsize);
    const int kOutputSize = kMaxNumOutputBbox * sizeof(Detection) / sizeof(float) + 1;
    cudaStream_t stream;
    float* gpu_buffers[2];
    float* cpu_output_buffer = nullptr;
    nvinfer1::IRuntime* runtime = nullptr;
    nvinfer1::ICudaEngine* engine = nullptr;
    nvinfer1::IExecutionContext* context = nullptr;

};

#endif // YOLOV5DET_H
